==> dataflow_421-0.2.1-py3-none-any.whl: dataflow/generator/algorithms/QuestionGenerator.py <==
import random
import os
import pandas as pd
try:
    from utils import  APIGenerator_aisuite, APIGenerator_request
    from utils.Prompts import QuestionSynthesisPrompt
except ImportError:
    from dataflow.generator.utils import  APIGenerator_aisuite, APIGenerator_request
    from dataflow.generator.utils.Prompts import QuestionSynthesisPrompt
from dataflow.utils.registry import GENERATOR_REGISTRY
import logging
from dataflow.utils.utils import get_logger
from dataflow.data import MyScaleStorage

@GENERATOR_REGISTRY.register()
class QuestionGenerator():
    def __init__(self, args):
        """
        Initialize the QuestionGenerator with the provided configuration.
        """
        self.config = args
        self.prompts = QuestionSynthesisPrompt()

        # Ensure the necessary configuration keys are provided
        if "db_name" in args.keys():
            self.storage = MyScaleStorage(args['db_port'], args['db_name'], args['table_name'])
            self.output_file = None
        else:
            self.input_file = args.get("input_file")
            self.output_file= args.get("output_file")
        self.input_key = self.config.get("input_key", "data") 
        self.read_key = self.config.get("read_key", "question")  # default key for question input
        self.num_prompts = self.config.get("num_prompts", 1)  # default number of prompts to use for generation
        self.read_min_score = self.config.get('read_min_score', 0.9)
        self.read_max_score = self.config.get('read_max_score', 2.0)
        self.eval_stage = self.config.get('eval_stage',1)
        # check if num_prompts is a valid number
        if self.num_prompts not in [0,1,2,3,4,5]:
            raise ValueError("num_prompts must be 0, 1, 2, 3, 4, or 5")

        # Validate that input_file and output_file are provided
        if "db_name" not in args.keys() and (not self.input_file or not self.output_file):
            raise ValueError("Both input_file and output_file must be specified in the config.")

        # Initialize the model
        self.model = self.__init_model__()
        self.logger = get_logger()
        

    def __init_model__(self):
        """
        Initialize the model generator based on the configuration.
        """
        generator_type = self.config.get("generator_type", "local").lower()

        # if generator_type == "local":
        #     return LocalModelGenerator(self.config)
        if generator_type == "aisuite":
            return APIGenerator_aisuite(self.config)
        elif generator_type == "request":
            return APIGenerator_request(self.config)
        else:
            raise ValueError(f"Invalid generator type: {generator_type}")

    def _reformat_prompt(self, dataframe):
        """
        Reformat the prompts in the dataframe to generate questions based on num_prompts.
        """
        # Check if read_key is in the dataframe
        if self.read_key not in dataframe.columns:
            key_list = dataframe.columns.tolist()
            raise ValueError(f"read_key: {self.read_key} not found in the dataframe. Available keys: {key_list}")

        # Predefined transformation options for diversity
        diversity_mode = [
            "1, 2, 3",
            "1, 2, 4",
            "1, 2, 5",
            "1, 4, 5",
            "1, 2, 3, 4, 5"
        ]

        formatted_prompts = []
        for question in dataframe[self.read_key]:
            if self.num_prompts == 0:
                formatted_prompts.append("")  # Skip generating for this question
            else:
                # Randomly choose the required number of transformations from diversity_mode
                selected_items = random.sample(diversity_mode, self.num_prompts)
                for selected_item in selected_items:
                    used_prompt = self.prompts.question_synthesis_prompt(selected_item, question)
                    formatted_prompts.append(used_prompt.strip())

        return formatted_prompts

    def _load_input(self):
        if hasattr(self, 'storage'):
            value_list = self.storage.read_code_json(
                [self.input_key], stage=self.eval_stage, format='SFT_Single', syn='syn_q', maxmin_scores=[{'min_score': self.read_min_score, 'max_score': self.read_max_score}]
            )
            # print(value_list)
            return pd.DataFrame([
                {**item['data'], 'id': str(item['id'])}
                for item in value_list
            ])
            # return pd.DataFrame([item['data'] for item in value_list])
        else:
            return pd.read_json(self.input_file, lines=True)

    def _write_output(self, save_path, dataframe, extractions):
        if hasattr(self, 'storage'):
            output_rows = dataframe.where(pd.notnull(dataframe), None).to_dict(orient="records")
            formatted_rows = [
                {
                    "id": row["id"],
                    "data": {k: v for k, v in row.items() if k != "id"}
                }
                for row in output_rows
            ]
            self.storage.write_code_json(formatted_rows, format="SFT_Single", syn="syn_q")
        else:
            output_dir = os.path.dirname(self.output_file)
            os.makedirs(output_dir, exist_ok=True)
            dataframe.to_json(save_path, orient="records", lines=True)

    def run(self):
        """
        Run the question generation process.
        """
        try:
            
            # Read the input file (jsonl format only)
            # dataframe = pd.read_json(self.input_file, lines=True)
            dataframe = self._load_input()
            if self.read_key not in dataframe.columns:
                raise ValueError(f"read_key: {self.read_key} not found in the dataframe. Available keys: {dataframe.columns.tolist()}")
            if "Synth_or_Input" in dataframe.columns:
                raise ValueError(f"Synth_or_Input is a reserved column name to show if the question is generated or not, please rename it")
            
            if self.num_prompts == 0:
                raise ValueError(f"num_prompts should not be 0.")
                # self.logger.info(f"num_prompts is 0, skip generation")
                # # dataframe.to_json(self.output_file, orient="records", lines=True, force_ascii=False)
                # if hasattr(self, 'storage'):
                #     self._write_output(self.output_file, dataframe, None)
                # self.logger.info(f"Generated questions saved to {self.output_file}")
                # return

            # Reformat the prompts for question generation
            formatted_prompts = self._reformat_prompt(dataframe)

            # Generate responses using the model
            responses = self.model.generate_text_from_input(formatted_prompts)

            # 将新生成的问题作为新的行添加到dataframe中，仍然填写到read_key中,这些行的其他列全部为空
            # new_rows = pd.DataFrame(columns=dataframe.columns)
            # new_rows[self.read_key] = responses
            repeat_count = self.num_prompts  # 每个seed生成几个
            expected_total = len(dataframe) * repeat_count

            if len(responses) != expected_total:
                raise ValueError(f"Expected {expected_total} responses (len(dataframe)={len(dataframe)} * {repeat_count}), but got {len(responses)}")

            # 展开原始id
            expanded_ids = [id_ for id_ in dataframe["id"].tolist() for _ in range(repeat_count)]

            # 构建新生成的行
            new_rows = pd.DataFrame({
                "id": expanded_ids,
                self.read_key: responses,
            })
            
            # if hasattr(self, 'storage'):
            #     self._write_output(self.output_file, new_rows, None)
            #     return 0
            
            new_rows["Synth_or_Input"] = "synth"
            dataframe["Synth_or_Input"] = "input"
            
            dataframe = pd.concat([dataframe, new_rows], ignore_index=True) # ignore_index=True 表示忽略原来的索引
        
            # Ensure output directory exists
            # output_dir = os.path.dirname(self.output_file)
            # os.makedirs(output_dir, exist_ok=True)

            # Save DataFrame to JSON file
            # 过滤self.read_key为空的数据
            dataframe = dataframe[dataframe[self.read_key].notna()]
            dataframe = dataframe[dataframe[self.read_key] != ""]
            # dataframe.to_json(self.output_file, orient="records", lines=True, force_ascii=False)
            self._write_output(self.output_file, dataframe, None)

            self.logger.info(f"Generated questions saved to {self.output_file}")

        except Exception as e:
            self.logger.error(f"[错误] 处理过程中发生异常: {e}")

==> dataflow_421-0.2.9-py3-none-any.whl: dataflow/generator/algorithms/QuestionGenerator.py <==
import random
import os
import pandas as pd
try:
    from utils import  APIGenerator_aisuite, APIGenerator_request
    from utils.Prompts import QuestionSynthesisPrompt
except ImportError:
    from dataflow.generator.utils import  APIGenerator_aisuite, APIGenerator_request
    from dataflow.generator.utils.Prompts import QuestionSynthesisPrompt
from dataflow.utils.registry import GENERATOR_REGISTRY
import logging
from dataflow.utils.utils import get_logger
from dataflow.data import MyScaleStorage, DatabaseConfig

@GENERATOR_REGISTRY.register()
class QuestionGenerator():
    def __init__(self, args):
        """
        Initialize the QuestionGenerator with the provided configuration.
        """
        self.config = args
        self.prompts = QuestionSynthesisPrompt()

        # Ensure the necessary configuration keys are provided
        use_db = args.get("use_db", False) or os.environ.get("USE_DB", "").lower() == "true"
        if use_db:
            db_config = DatabaseConfig(
                host=os.environ.get('MYSCALE_HOST', 'localhost'),
                port=int(os.environ.get('MYSCALE_PORT', '9000')),
                db_name=os.environ.get('MYSCALE_DATABASE', 'dataflow'),
                table_name=os.environ.get('MYSCALE_TABLE_NAME', ''),
                username=os.environ.get('MYSCALE_USER', ''),
                password=os.environ.get('MYSCALE_PASSWORD', '')
            )
            self.storage = MyScaleStorage(db_config)
            self.input_file = None
            self.output_file = None
        else:
            self.input_file = args.get("input_file")
            self.output_file = args.get("output_file")
        self.input_key = self.config.get("input_key", "data") 
        self.read_key = self.config.get("read_key", "question")  # default key for question input
        self.num_prompts = self.config.get("num_prompts", 1)  # default number of prompts to use for generation
        self.read_min_score = self.config.get('read_min_score', 0.9)
        self.read_max_score = self.config.get('read_max_score', 2.0)
        self.eval_stage = self.config.get('eval_stage',1)
        # check if num_prompts is a valid number
        if self.num_prompts not in [0,1,2,3,4,5]:
            raise ValueError("num_prompts must be 0, 1, 2, 3, 4, or 5")

        # Validate that input_file and output_file are provided
        if "db_name" not in args.keys() and (not self.input_file or not self.output_file):
            raise ValueError("Both input_file and output_file must be specified in the config.")

        # Initialize the model
        self.model = self.__init_model__()
        self.logger = get_logger()
        

    def __init_model__(self):
        """
        Initialize the model generator based on the configuration.
        """
        generator_type = self.config.get("generator_type", "local").lower()

        # if generator_type == "local":
        #     return LocalModelGenerator(self.config)
        if generator_type == "aisuite":
            return APIGenerator_aisuite(self.config)
        elif generator_type == "request":
            return APIGenerator_request(self.config)
        else:
            raise ValueError(f"Invalid generator type: {generator_type}")

    def _reformat_prompt(self, dataframe):
        """
        Reformat the prompts in the dataframe to generate questions based on num_prompts.
        """
        # Check if read_key is in the dataframe
        if self.read_key not in dataframe.columns:
            key_list = dataframe.columns.tolist()
            raise ValueError(f"read_key: {self.read_key} not found in the dataframe. Available keys: {key_list}")

        # Predefined transformation options for diversity
        diversity_mode = [
            "1, 2, 3",
            "1, 2, 4",
            "1, 2, 5",
            "1, 4, 5",
            "1, 2, 3, 4, 5"
        ]

        formatted_prompts = []
        for question in dataframe[self.read_key]:
            if self.num_prompts == 0:
                formatted_prompts.append("")  # Skip generating for this question
            else:
                # Randomly choose the required number of transformations from diversity_mode
                selected_items = random.sample(diversity_mode, self.num_prompts)
                for selected_item in selected_items:
                    used_prompt = self.prompts.question_synthesis_prompt(selected_item, question)
                    formatted_prompts.append(used_prompt.strip())

        return formatted_prompts

    def _load_input(self):
        if hasattr(self, 'storage'):
            value_list = self.storage.read_code_json(
                [self.input_key], stage=self.eval_stage, format='SFT_Single', syn='syn_q', maxmin_scores=[{'min_score': self.read_min_score, 'max_score': self.read_max_score}]
            )
            # print(value_list)
            return pd.DataFrame([
                {**item['data'], 'id': str(item['id'])}
                for item in value_list
            ])
            # return pd.DataFrame([item['data'] for item in value_list])
        else:
            return pd.read_json(self.input_file, lines=True)

    def _write_output(self, save_path, dataframe, extractions):
        if hasattr(self, 'storage'):
            output_rows = dataframe.where(pd.notnull(dataframe), None).to_dict(orient="records")
            formatted_rows = [
                {
                    "id": row["id"],
                    "data": {k: v for k, v in row.items() if k != "id"}
                }
                for row in output_rows
            ]
            self.storage.write_code_json(formatted_rows, format="SFT_Single", syn="syn_q")
        else:
            output_dir = os.path.dirname(self.output_file)
            os.makedirs(output_dir, exist_ok=True)
            dataframe.to_json(save_path, orient="records", lines=True)

    def run(self):
        """
        Run the question generation process.
        """
        try:
            
            # Read the input file (jsonl format only)
            # dataframe = pd.read_json(self.input_file, lines=True)
            dataframe = self._load_input()
            if self.read_key not in dataframe.columns:
                raise ValueError(f"read_key: {self.read_key} not found in the dataframe. Available keys: {dataframe.columns.tolist()}")
            if "Synth_or_Input" in dataframe.columns:
                raise ValueError(f"Synth_or_Input is a reserved column name to show if the question is generated or not, please rename it")
            
            if self.num_prompts == 0:
                raise ValueError(f"num_prompts should not be 0.")
                # self.logger.info(f"num_prompts is 0, skip generation")
                # # dataframe.to_json(self.output_file, orient="records", lines=True, force_ascii=False)
                # if hasattr(self, 'storage'):
                #     self._write_output(self.output_file, dataframe, None)
                # self.logger.info(f"Generated questions saved to {self.output_file}")
                # return

            # Reformat the prompts for question generation
            formatted_prompts = self._reformat_prompt(dataframe)

            # Generate responses using the model
            responses = self.model.generate_text_from_input(formatted_prompts)

            # 将新生成的问题作为新的行添加到dataframe中，仍然填写到read_key中,这些行的其他列全部为空
            # new_rows = pd.DataFrame(columns=dataframe.columns)
            # new_rows[self.read_key] = responses
            repeat_count = self.num_prompts  # 每个seed生成几个
            expected_total = len(dataframe) * repeat_count

            if len(responses) != expected_total:
                raise ValueError(f"Expected {expected_total} responses (len(dataframe)={len(dataframe)} * {repeat_count}), but got {len(responses)}")

            # 展开原始id
            expanded_ids = [id_ for id_ in dataframe["id"].tolist() for _ in range(repeat_count)]

            # 构建新生成的行
            new_rows = pd.DataFrame({
                "id": expanded_ids,
                self.read_key: responses,
            })
            
            # if hasattr(self, 'storage'):
            #     self._write_output(self.output_file, new_rows, None)
            #     return 0
            
            new_rows["Synth_or_Input"] = "synth"
            dataframe["Synth_or_Input"] = "input"
            
            dataframe = pd.concat([dataframe, new_rows], ignore_index=True) # ignore_index=True 表示忽略原来的索引
        
            # Ensure output directory exists
            # output_dir = os.path.dirname(self.output_file)
            # os.makedirs(output_dir, exist_ok=True)

            # Save DataFrame to JSON file
            # 过滤self.read_key为空的数据
            dataframe = dataframe[dataframe[self.read_key].notna()]
            dataframe = dataframe[dataframe[self.read_key] != ""]
            # dataframe.to_json(self.output_file, orient="records", lines=True, force_ascii=False)
            self._write_output(self.output_file, dataframe, None)

            self.logger.info(f"Generated questions saved to {self.output_file}")

        except Exception as e:
            self.logger.error(f"[错误] 处理过程中发生异常: {e}")
